{
  Beispielprogramm fuer die Textopia-Toolbox
  Kapitel 5.2
}
PROGRAM demo2;
USES TSTRING,TIO,TMAIN;

CONST
  forest_id   = 10;
  sword_id    = 11;
  rucksack_id = 12;

VAR
  room   : PROOM;
  item   : PItem;
  lock   : PLock;
  player : PPLAYER;
  game   : PGAME;

BEGIN
  Prepare('grab',t);
  {
    --- Wald ---
  }
  NEW(room,Init(forest_id,'+W%ald#er'));
  WITH room^ DO
  BEGIN
    AddText('Fast undurchdringliches Unterholz läßt Sie nur mühsam '+
            'weiterkommen. Weiter westlich scheinen die Bäume weniger '+
            'dicht beieinander zu stehen.\n');
  END;
  {
    --- Spieler ---
  }
  NEW(player,Init(horse_id,sie,male,1,10));
  {
    --- Schwert ---
  }
  NEW(item,Init(sword_id,'Schwert#er',NIL,t,1,0));
  WITH item^ DO
  BEGIN
    SetAttrib(takeable_at,t);
    AddText('Das Schwert hat der Waffenschmied Ihnen kurz vor Ihrer '+
            'Abreise gegeben. Es schimmert und ist sehr scharf. Den '+
            'Schmied zu überreden, Ihnen das Schwert anzuschreiben, weil '+
            'Sie die nötigen Taler erst noch in den Gräbern finden '+
            'müßten, war nicht einfach.\n');
  END;
  {
    --- Rucksack ---
  }
  NEW(item,Init(rucksack_id,'+Rucks%ack#e',NIL,t,1,5));
  WITH item^ DO
  BEGIN
    SetAttrib(takeable_at,t);
  END;
  NEW(lock,Init(item,t,t,closed,NIL));
  {
    --- Spiel ---
  }
  NEW(game,Init(t,f,t,4));
  WITH game^ DO
  BEGIN
    AddProlog('Das Zwergengrab\nEine Texttopia-Demo von Oliver Berse\n\n'+
              'Drei Tage schon durchstreifen Sie abseits der Straßen den '+
              'alten Wald. Hier irgendwo liegen die mit Schätzen gefüllten '+
              'Gräber der Zwergenkönige, die einst diese Länder '+
              'beherrschten.\n\n');
    Run;
    Done;
  END;
END.
