{
  Beispielprogramm fuer die Textopia-Toolbox
  Kapitel 4.1
}
PROGRAM demo1;
USES TSTRING,TIO,TMAIN;

CONST
  forest_id = 10;

VAR
  room   : PROOM;
  player : PPLAYER;
  game   : PGAME;

BEGIN
  Prepare('grab',t);
  {
    --- Wald ---
  }
  NEW(room,Init(forest_id,'+W%ald#er'));
  WITH room^ DO
  BEGIN
    AddText('Fast undurchdringliches Unterholz läßt Sie nur mühsam '+
            'weiterkommen. Weiter westlich scheinen die Bäume weniger '+
            'dicht beieinander zu stehen.');
  END;
  {
    --- Spieler ---
  }
  NEW(player,Init(forest_id,sie,male,1,3));
  {
    --- Spiel ---
  }
  NEW(game,Init(t,f,t,4));
  WITH game^ DO
  BEGIN
    AddProlog('Das Zwergengrab\nEine Texttopia-Demo von Oliver Berse\n\n'+
              'Drei Tage schon durchstreifen Sie abseits der Straßen den '+
              'alten Wald. Hier irgendwo liegen die mit Schätzen gefüllten '+
              'Gräber der Zwergenkönige, die einst diese Länder '+
              'beherrschten.\n\n');
    Run;
    Done;
  END;
END.
